/*
Copyright 2010-2011 Zhengmao HU (James)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package net.sf.jabb.util.text.word;

import java.util.Map;
import java.util.TreeMap;

/**
 * Text Analyzer; Result of the analysis will be hold in {@link AnalyzedText}.<br>
 * ıĽ{@link AnalyzedText}С
 * 
 * @author Zhengmao HU (James)
 *
 */
public abstract class TextAnalyzer {
	/**
	 * ʹcom.chenlb.mmseg4j.SimpleSegзִ
	 */
	static public final int TYPE_MMSEG_SIMPLE = 1;
	/**
	 * ʹcom.chenlb.mmseg4j.MaxWordSegзִ
	 */
	static public final int TYPE_MMSEG_MAXWORD = 2;
	/**
	 * ʹcom.chenlb.mmseg4j.ComplexSegзִ
	 */
	static public final int TYPE_MMSEG_COMPLEX = 3;
	/**
	 * ʹKeywordMatcherԶֵзִʣУвƣ
	 */
	static public final int TYPE_FAST = 4;
	
	protected String dictionaryPath;
	protected Map<String, ? extends Object> keywordDefinitions;
	protected TreeMap<Integer, ? extends Object> lengthDefinitions;
	
	/**
	 * Create an instance of TextAnalyzer.<br>
	 * һıʵ
	 * 
	 * @param type				{@link #TYPE_MMSEG_SIMPLE} | {@link #TYPE_MMSEG_COMPLEX} | {@link #TYPE_MMSEG_MAXWORD} | {@link #TYPE_FAST}
	 * @param dictionaryPath	ֵļ·Ϊnullʾʹȱʡλõֵļ
	 * @param keywordDefinitions	ؼֵĶ
	 * @param lengthDefinitions		ı
	 * @return	A new instance of TextAnalyzer.<br>TextAnalyzerһʵ
	 */
	static public TextAnalyzer createInstance(int type, 
			String dictionaryPath, 
			Map<String, ? extends Object> keywordDefinitions, 
			Map<Integer, ? extends Object> lengthDefinitions){
		switch(type){
		case TYPE_MMSEG_COMPLEX:
		case TYPE_MMSEG_MAXWORD:
		case TYPE_MMSEG_SIMPLE:
			return new MmsegTextAnalyzer(type, dictionaryPath, keywordDefinitions, lengthDefinitions);
		case TYPE_FAST:
			return new FastTextAnalyzer(dictionaryPath, keywordDefinitions, lengthDefinitions);
		default:
			throw new IllegalArgumentException("Not supported TextAnalyzer type: " + type);
		}
	}
	
	/**
	 * Create an instance of TextAnalyzer.<br>
	 * һıʵ
	 * 
	 * @param type				{@link #TYPE_MMSEG_SIMPLE} | {@link #TYPE_MMSEG_COMPLEX} | {@link #TYPE_MMSEG_MAXWORD} | {@link #TYPE_FAST}
	 * @param keywordDefinitions	ؼֵĶ
	 * @param lengthDefinitions		ı
	 * @return	A new instance of TextAnalyzer.<br>TextAnalyzerһʵ
	 */
	static public TextAnalyzer createInstance(int type, 
			Map<String, ? extends Object> keywordDefinitions, 
			Map<Integer, ? extends Object> lengthDefinitions){
		return createInstance(type, null, keywordDefinitions, lengthDefinitions);
	}

	/**
	 * Create an instance of TextAnalyzer.<br>
	 * һıʵ
	 * 
	 * @param type				{@link #TYPE_MMSEG_SIMPLE} | {@link #TYPE_MMSEG_COMPLEX} | {@link #TYPE_MMSEG_MAXWORD} | {@link #TYPE_FAST}
	 * @return	A new instance of TextAnalyzer.<br>TextAnalyzerһʵ
	 */
	static public TextAnalyzer createInstance(int type){
		return createInstance(type, null, null, null);
	}

	
	/**
	 * Constructor that will be used internally.<br>
	 * ڲʹõĹ췽
	 * @param dictionaryPath		ֵļ·
	 * @param keywordDefinitions	ؼֵĶ
	 * @param lengthDefinitions		ı
	 */
	protected TextAnalyzer(String dictionaryPath, 
			Map<String, ? extends Object> keywordDefinitions, 
			Map<Integer, ? extends Object> lengthDefinitions){
		this.dictionaryPath = dictionaryPath;
		this.keywordDefinitions = keywordDefinitions;
		this.lengthDefinitions = lengthDefinitions == null ? 
				null : new TreeMap<Integer, Object>(lengthDefinitions);
	}
	
	/**
	 * ıз
	 * @param text	ı
	 * @param lazy	ǷӳٷνӳֱָõʱŽʵԷ
	 * @return		
	 */
	public AnalyzedText analyze(String text, boolean lazy){
		AnalyzedText result = new AnalyzedText(this, text);
		if (!lazy){
			analyzeLength(result);
			analyzeWords(result);
			analyzeKeywords(result);
		}
		return result;
	}
	
	/**
	 * ı̷lazyʽ
	 * @param text	ı
	 * @return		
	 */
	public AnalyzedText analyze(String text){
		return analyze(text, false);
	}
	
	/**
	 * ıȷıĳĸ䡣
	 * @param aText		ȡԭʼıԼŽ
	 */
	void analyzeLength(AnalyzedText aText){
		if (lengthDefinitions != null){
			for (int l: lengthDefinitions.keySet()){
				if (aText.getText().length() <= l){
					aText.setLengthCategory(lengthDefinitions.get(l));
					break;
				}
			}
		}
	}
	
	/**
	 * зִʡıָΪһʻ֡
	 * @param aText		ȡԭʼıԼŽ
	 */
	abstract void analyzeWords(AnalyzedText aText);
	
	/**
	 * йؼƥ䡣
	 * @param aText	ȡԭʼıԼŽ
	 */
	abstract void analyzeKeywords(AnalyzedText aText);
	
	/**
	 * ¼ݡ
	 * @param keywordDefinitions	ؼʶ
	 * @param lengthDefinitions		ı
	 */
	void reloadDefinitions(
			Map<String, ? extends Object> keywordDefinitions, 
			Map<Integer, ? extends Object> lengthDefinitions){
			this.lengthDefinitions = lengthDefinitions == null ?
					null : new TreeMap<Integer, Object>(lengthDefinitions);
	}
		
}
